
#include <stdio.h>
#include <cog.h>
#include <propeller.h>
#include "Servo32_Highres.h"


static long	FastPins, SlowPins;
static long Cycles;
static long ServoData[32];		//Servo Pulse Width information
static long Scale;


int Servo32_Driver_Start( void * );


void Servo32_Start(void)
{
	Servo32_Driver_Start(&FastPins);
}


void Servo32_Init( int fastRate )
{
	MasterLoopDelay = 80000000 / fastRate;
	SlowUpdateCounter = (80000000 / 50) / MasterLoopDelay;

	//10 clocks is the smallest amount we can wait - everything else is based on that.
	//If you're using a different clock speed, your center point will likely need to be adjusted
	Scale = 10;

	FastPins = 0;
	SlowPins = 0;
}


//Set a PIN index as a high-speed output (250Hz)  
void Servo32_AddFastPin( int Pin )
{
	FastPins |= (1<<Pin);                                            
	SlowPins |= (1<<Pin);	//Fast pins are also updated during the "slow" cycle                                              
}


//Set a PIN index as a standard-speed output (50Hz)  
void Servo32_AddSlowPin( int Pin )
{
	SlowPins |= (1<<Pin);	//Slow pins are ONLY updated during the "slow" cycle
}


void Servo32_Set( int ServoPin, int Width )		// Set Servo value as a raw delay, in 10 clock increments
{
	ServoData[ServoPin] = Width * Scale;		// Servo widths are set in 10ths of a uS, so 8000 = min, 12000 = mid, 16000 = max
}

void Servo32_SetRC( int ServoPin, int Width )	// Set Servo value signed, assuming 12000 is your center
{
	ServoData[ServoPin] = 12000 + Width * Scale;	// Servo widths are set in 10ths of a uS, so -4000 min, 0 = mid, +4000 = max
}

int Servo32_GetCycles(void)
{
	return Cycles;
}

